@@ -20,13 +20,20 @@
  */
 package org.apache.bookkeeper.http.vertx;
 
+import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
+import com.google.common.io.Files;
+import io.vertx.ext.web.handler.BodyHandler;
 import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
+import java.io.OutputStreamWriter;
+import java.io.PrintWriter;
 import java.net.HttpURLConnection;
 import java.net.URL;
 import java.nio.charset.StandardCharsets;
@@ -97,6 +104,22 @@ public class TestVertxHttpServer {
         httpServer.stopServer();
     }
 
+    @Test
+    public void testArbitraryFileUpload() throws IOException {
+        VertxHttpServer httpServer = new VertxHttpServer();
+        HttpServiceProvider httpServiceProvider = NullHttpServiceProvider.getInstance();
+        httpServer.initialize(httpServiceProvider);
+        assertTrue(httpServer.startServer(0));
+        int port = httpServer.getListeningPort();
+        File tempFile = File.createTempFile("test-" + System.currentTimeMillis(), null);
+        Files.asCharSink(tempFile, StandardCharsets.UTF_8).write(TestVertxHttpServer.class.getName());
+        String[] filenamesBeforeUploadRequest = listFiles(BodyHandler.DEFAULT_UPLOADS_DIRECTORY);
+        HttpResponse httpResponse = sendFile(getUrl(port, HttpRouter.BOOKIE_INFO), tempFile);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), httpResponse.responseCode);
+        assertArrayEquals(filenamesBeforeUploadRequest, listFiles(BodyHandler.DEFAULT_UPLOADS_DIRECTORY));
+        httpServer.stopServer();
+    }
+
     private HttpResponse send(String url, HttpServer.Method method) throws IOException {
         return send(url, method, "");
     }
@@ -131,6 +154,59 @@ public class TestVertxHttpServer {
         return new HttpResponse(responseCode, response.toString());
     }
 
+    private HttpResponse sendFile(String url, File file) throws IOException {
+        URL obj = new URL(url);
+        HttpURLConnection con = (HttpURLConnection) obj.openConnection();
+        String boundary = "---------------------------" + System.currentTimeMillis();
+        // optional, default is GET
+        con.setRequestMethod("POST");
+        con.setDoOutput(true);
+        con.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
+        try (
+                OutputStream outputStream = con.getOutputStream();
+                PrintWriter writer = new PrintWriter(new OutputStreamWriter(outputStream, StandardCharsets.UTF_8),
+                        true);
+                FileInputStream fileInputStream = new FileInputStream(file);
+        ) {
+            writer.append("--" + boundary).append("\r\n");
+            writer.append("Content-Disposition: form-data; name=\"file\"; filename=\"file.txt\"").append("\r\n");
+            writer.append("Content-Type: text/plain").append("\r\n");
+            writer.append("\r\n");
+
+            byte[] buffer = new byte[4096];
+            int bytesRead;
+            while ((bytesRead = fileInputStream.read(buffer)) != -1) {
+                outputStream.write(buffer, 0, bytesRead);
+            }
+
+            writer.append("\r\n");
+            writer.append("--" + boundary + "--").append("\r\n");
+        }
+        int responseCode = con.getResponseCode();
+        StringBuilder response = new StringBuilder();
+        BufferedReader in = null;
+        try {
+            in = new BufferedReader(new InputStreamReader(con.getInputStream()));
+            String inputLine;
+            while ((inputLine = in.readLine()) != null) {
+                response.append(inputLine);
+            }
+        } finally {
+            if (in != null) {
+                in.close();
+            }
+        }
+        return new HttpResponse(responseCode, response.toString());
+    }
+
+    private String[] listFiles(String directory) {
+        File directoryFile = new File(directory);
+        if (!directoryFile.exists() || !directoryFile.isDirectory()) {
+            return new String[0];
+        }
+        return directoryFile.list();
+    }
+
     private String getUrl(int port, String path) {
         return "http://localhost:" + port + path;
     }
