@@ -104,7 +104,6 @@ import org.apache.commons.configuration.CompositeConfiguration;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.zookeeper.KeeperException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -477,6 +476,8 @@ public class BookieShell implements Tool {
             opts.addOption("l", "ledger", true, "Recover a specific ledger");
             opts.addOption("sk", "skipOpenLedgers", false, "Skip recovering open ledgers");
             opts.addOption("d", "deleteCookie", false, "Delete cookie node for the bookie.");
+            opts.addOption("sku", "skipUnrecoverableLedgers", false, "Skip unrecoverable ledgers.");
+            opts.addOption("rate", "replicationRate", false, "Replication rate by bytes");
         }
 
         @Override
@@ -509,8 +510,10 @@ public class BookieShell implements Tool {
             boolean force = cmdLine.hasOption("f");
             boolean skipOpenLedgers = cmdLine.hasOption("sk");
             boolean removeCookies = !dryrun && cmdLine.hasOption("d");
+            boolean skipUnrecoverableLedgers = cmdLine.hasOption("sku");
 
             Long ledgerId = getOptionLedgerIdValue(cmdLine, "ledger", -1);
+            int replicationRate = getOptionIntValue(cmdLine, "replicationRate", -1);
 
             RecoverCommand cmd = new RecoverCommand();
             RecoverCommand.RecoverFlags flags = new RecoverCommand.RecoverFlags();
@@ -519,8 +522,10 @@ public class BookieShell implements Tool {
             flags.dryRun(dryrun);
             flags.force(force);
             flags.ledger(ledgerId);
+            flags.replicateRate(replicationRate);
             flags.skipOpenLedgers(skipOpenLedgers);
             flags.query(query);
+            flags.skipUnrecoverableLedgers(skipUnrecoverableLedgers);
             boolean result = cmd.apply(bkConf, flags);
             return (result) ? 0 : -1;
         }
@@ -1042,10 +1047,11 @@ public class BookieShell implements Tool {
 
             long logId;
             try {
-                logId = Long.parseLong(leftArgs[0]);
+                logId = Long.parseLong(leftArgs[0], 16);
                 flags.logId(logId);
             } catch (NumberFormatException nfe) {
                 flags.logFilename(leftArgs[0]);
+                flags.logId(-1);
             }
             cmd.apply(bkConf, flags);
             return 0;
@@ -1900,7 +1906,7 @@ public class BookieShell implements Tool {
                                 underreplicationManager.setReplicasCheckCTime(time);
                             }
                         }
-                    } catch (InterruptedException | KeeperException | ReplicationException e) {
+                    } catch (InterruptedException | ReplicationException e) {
                         LOG.error("Exception while trying to reset last run time ", e);
                         return -1;
                     }
