@@ -137,6 +137,8 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     private final long maxReadAheadBytesSize;
 
+    private final boolean singleLedgerDirs;
+
     public SingleDirectoryDbLedgerStorage(ServerConfiguration conf, LedgerManager ledgerManager,
             LedgerDirsManager ledgerDirsManager, LedgerDirsManager indexDirsManager, StateManager stateManager,
             CheckpointSource checkpointSource, Checkpointer checkpointer, StatsLogger statsLogger,
@@ -152,6 +154,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         this.writeCacheMaxSize = writeCacheSize;
         this.writeCache = new WriteCache(allocator, writeCacheMaxSize / 2);
         this.writeCacheBeingFlushed = new WriteCache(allocator, writeCacheMaxSize / 2);
+        this.singleLedgerDirs = conf.getLedgerDirs().length == 1;
 
         this.checkpointSource = checkpointSource;
 
@@ -266,10 +269,13 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     @Override
     public boolean isFenced(long ledgerId) throws IOException {
+        boolean isFenced = ledgerIndex.get(ledgerId).getFenced();
+
         if (log.isDebugEnabled()) {
-            log.debug("isFenced. ledger: {}", ledgerId);
+            log.debug("ledger: {}, isFenced: {}.", ledgerId, isFenced);
         }
-        return ledgerIndex.get(ledgerId).getFenced();
+
+        return isFenced;
     }
 
     @Override
@@ -348,6 +354,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     private void triggerFlushAndAddEntry(long ledgerId, long entryId, ByteBuf entry)
             throws IOException, BookieException {
+        long throttledStartTime = MathUtils.nowInNano();
         dbLedgerStorageStats.getThrottledWriteRequests().inc();
         long absoluteTimeoutNanos = System.nanoTime() + maxThrottleTimeNanos;
 
@@ -371,6 +378,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             try {
                 if (writeCache.put(ledgerId, entryId, entry)) {
                     // We succeeded in putting the entry in write cache in the
+                    recordSuccessfulEvent(dbLedgerStorageStats.getThrottledWriteStats(), throttledStartTime);
                     return;
                 }
             } finally {
@@ -388,6 +396,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
         // Timeout expired and we weren't able to insert in write cache
         dbLedgerStorageStats.getRejectedWriteRequests().inc();
+        recordFailedEvent(dbLedgerStorageStats.getThrottledWriteStats(), throttledStartTime);
         throw new OperationRejectedException();
     }
 
@@ -445,15 +454,27 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
         // Read from main storage
         long entryLocation;
+        long locationIndexStartNano = MathUtils.nowInNano();
         try {
             entryLocation = entryLocationIndex.getLocation(ledgerId, entryId);
             if (entryLocation == 0) {
+                recordFailedEvent(dbLedgerStorageStats.getReadEntryStats(), startTime);
                 throw new NoEntryException(ledgerId, entryId);
             }
+        } finally {
+            dbLedgerStorageStats.getReadFromLocationIndexTime().registerSuccessfulEvent(
+                MathUtils.elapsedNanos(locationIndexStartNano), TimeUnit.NANOSECONDS);
+        }
+
+        long readEntryStartNano = MathUtils.nowInNano();
+        try {
             entry = entryLogger.readEntry(ledgerId, entryId, entryLocation);
         } catch (NoEntryException e) {
             recordFailedEvent(dbLedgerStorageStats.getReadEntryStats(), startTime);
             throw e;
+        } finally {
+            dbLedgerStorageStats.getReadFromEntryLogTime().registerSuccessfulEvent(
+                MathUtils.elapsedNanos(readEntryStartNano), TimeUnit.NANOSECONDS);
         }
 
         readCache.put(ledgerId, entryId, entry);
@@ -628,21 +649,6 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
             ledgerIndex.flush();
 
-            cleanupExecutor.execute(() -> {
-                // There can only be one single cleanup task running because the cleanupExecutor
-                // is single-threaded
-                try {
-                    if (log.isDebugEnabled()) {
-                        log.debug("Removing deleted ledgers from db indexes");
-                    }
-
-                    entryLocationIndex.removeOffsetFromDeletedLedgers();
-                    ledgerIndex.removeDeletedLedgers();
-                } catch (Throwable t) {
-                    log.warn("Failed to cleanup db indexes", t);
-                }
-            });
-
             lastCheckpoint = thisCheckpoint;
 
             // Discard all the entry from the write cache, since they're now persisted
@@ -665,6 +671,21 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             throw new IOException(e);
         } finally {
             try {
+                cleanupExecutor.execute(() -> {
+                    // There can only be one single cleanup task running because the cleanupExecutor
+                    // is single-threaded
+                    try {
+                        if (log.isDebugEnabled()) {
+                            log.debug("Removing deleted ledgers from db indexes");
+                        }
+
+                        entryLocationIndex.removeOffsetFromDeletedLedgers();
+                        ledgerIndex.removeDeletedLedgers();
+                    } catch (Throwable t) {
+                        log.warn("Failed to cleanup db indexes", t);
+                    }
+                });
+
                 isFlushOngoing.set(false);
             } finally {
                 flushMutex.unlock();
@@ -699,7 +720,9 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
     public void flush() throws IOException {
         Checkpoint cp = checkpointSource.newCheckpoint();
         checkpoint(cp);
-        checkpointSource.checkpointComplete(cp, true);
+        if (singleLedgerDirs) {
+            checkpointSource.checkpointComplete(cp, true);
+        }
     }
 
     @Override
@@ -737,9 +760,30 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     @Override
     public void updateEntriesLocations(Iterable<EntryLocation> locations) throws IOException {
-        // Trigger a flush to have all the entries being compacted in the db storage
-        flush();
+        // Before updating the DB with the new location for the compacted entries, we need to
+        // make sure that there is no ongoing flush() operation.
+        // If there were a flush, we could have the following situation, which is highly
+        // unlikely though possible:
+        // 1. Flush operation has written the write-cache content into entry-log files
+        // 2. The DB location index is not yet updated
+        // 3. Compaction is triggered and starts compacting some of the recent files
+        // 4. Compaction will write the "new location" into the DB
+        // 5. The pending flush() will overwrite the DB with the "old location", pointing
+        //    to a file that no longer exists
+        //
+        // To avoid this race condition, we need that all the entries that are potentially
+        // included in the compaction round to have all the indexes already flushed into
+        // the DB.
+        // The easiest lightweight way to achieve this is to wait for any pending
+        // flush operation to be completed before updating the index with the compacted
+        // entries, by blocking on the flushMutex.
+        flushMutex.lock();
+        flushMutex.unlock();
 
+        // We don't need to keep the flush mutex locked here while updating the DB.
+        // It's fine to have a concurrent flush operation at this point, because we
+        // know that none of the entries being flushed was included in the compaction
+        // round that we are dealing with.
         entryLocationIndex.updateLocations(locations);
     }
 
@@ -864,6 +908,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             }
         }
 
+        ledgerIndex.flush();
         batch.flush();
         batch.close();
 
