@@ -34,6 +34,15 @@ public interface BookieWatcher {
     Set<BookieId> getReadOnlyBookies() throws BKException;
     BookieAddressResolver getBookieAddressResolver();
 
+    /**
+     * Determine if a bookie should be considered unavailable.
+     *
+     * @param id
+     *          Bookie to check
+     * @return whether or not the given bookie is unavailable
+     */
+    boolean isBookieUnavailable(BookieId id);
+
     /**
      * Create an ensemble with given <i>ensembleSize</i> and <i>writeQuorumSize</i>.
      *
