@@ -20,6 +20,7 @@ package org.apache.bookkeeper.client;
 import io.netty.util.HashedWheelTimer;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -36,6 +37,7 @@ import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.net.BookieNode;
 import org.apache.bookkeeper.net.DNSToSwitchMapping;
 import org.apache.bookkeeper.net.NetworkTopology;
+import org.apache.bookkeeper.net.NetworkTopologyImpl;
 import org.apache.bookkeeper.net.Node;
 import org.apache.bookkeeper.net.NodeBase;
 import org.apache.bookkeeper.proto.BookieAddressResolver;
@@ -85,25 +87,6 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
         address2Region = new ConcurrentHashMap<BookieId, String>();
     }
 
-    protected String getRegion(BookieId addr) {
-        String region = address2Region.get(addr);
-        if (null == region) {
-            String networkLocation = resolveNetworkLocation(addr);
-            if (NetworkTopology.DEFAULT_REGION_AND_RACK.equals(networkLocation)) {
-                region = UNKNOWN_REGION;
-            } else {
-                String[] parts = networkLocation.split(NodeBase.PATH_SEPARATOR_STR);
-                if (parts.length <= 1) {
-                    region = UNKNOWN_REGION;
-                } else {
-                    region = parts[1];
-                }
-            }
-            address2Region.putIfAbsent(addr, region);
-        }
-        return region;
-    }
-
     protected String getLocalRegion(BookieNode node) {
         if (null == node || null == node.getAddr()) {
             return UNKNOWN_REGION;
@@ -111,6 +94,29 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
         return getRegion(node.getAddr());
     }
 
+    protected String getRegion(BookieId addr) {
+        String region = address2Region.get(addr);
+        if (null == region) {
+            region = parseBookieRegion(addr);
+            address2Region.putIfAbsent(addr, region);
+        }
+        return region;
+    }
+
+    protected String parseBookieRegion(BookieId addr) {
+        String networkLocation = resolveNetworkLocation(addr);
+        if (NetworkTopology.DEFAULT_REGION_AND_RACK.equals(networkLocation)) {
+            return UNKNOWN_REGION;
+        } else {
+            String[] parts = networkLocation.split(NodeBase.PATH_SEPARATOR_STR);
+            if (parts.length <= 1) {
+                return UNKNOWN_REGION;
+            } else {
+                return parts[1];
+            }
+        }
+    }
+
     @Override
     public void handleBookiesThatLeft(Set<BookieId> leftBookies) {
         super.handleBookiesThatLeft(leftBookies);
@@ -129,6 +135,7 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
             BookieNode node = createBookieNode(addr);
             topology.add(node);
             knownBookies.put(addr, node);
+            historyBookies.put(addr, node);
             String region = getLocalRegion(node);
             if (null == perRegionPlacement.get(region)) {
                 perRegionPlacement.put(region, new RackawareEnsemblePlacementPolicy()
@@ -162,6 +169,57 @@ public class RegionAwareEnsemblePlacementPolicy extends RackawareEnsemblePlaceme
         }
     }
 
+    @Override
+    public void onBookieRackChange(List<BookieId> bookieAddressList) {
+        rwLock.writeLock().lock();
+        try {
+            bookieAddressList.forEach(bookieAddress -> {
+                try {
+                    BookieNode node = knownBookies.get(bookieAddress);
+                    if (node != null) {
+                        // refresh the rack info if its a known bookie
+                        BookieNode newNode = createBookieNode(bookieAddress);
+                        if (!newNode.getNetworkLocation().equals(node.getNetworkLocation())) {
+                            topology.remove(node);
+                            topology.add(newNode);
+                            knownBookies.put(bookieAddress, newNode);
+                            historyBookies.put(bookieAddress, newNode);
+                        }
+                        //Handle per region placement policy.
+                        String oldRegion = getRegion(bookieAddress);
+                        String newRegion = parseBookieRegion(newNode.getAddr());
+                        if (oldRegion.equals(newRegion)) {
+                            TopologyAwareEnsemblePlacementPolicy regionPlacement = perRegionPlacement.get(oldRegion);
+                            regionPlacement.onBookieRackChange(Collections.singletonList(bookieAddress));
+                        } else {
+                            address2Region.put(bookieAddress, newRegion);
+                            TopologyAwareEnsemblePlacementPolicy oldRegionPlacement = perRegionPlacement.get(oldRegion);
+                            oldRegionPlacement.handleBookiesThatLeft(Collections.singleton(bookieAddress));
+                            TopologyAwareEnsemblePlacementPolicy newRegionPlacement = perRegionPlacement.get(
+                                    newRegion);
+                            if (newRegionPlacement == null) {
+                                newRegionPlacement = new RackawareEnsemblePlacementPolicy()
+                                        .initialize(dnsResolver, timer, this.reorderReadsRandom,
+                                                this.stabilizePeriodSeconds, this.reorderThresholdPendingRequests,
+                                                this.isWeighted, this.maxWeightMultiple,
+                                                this.minNumRacksPerWriteQuorum, this.enforceMinNumRacksPerWriteQuorum,
+                                                this.ignoreLocalNodeInPlacementPolicy, statsLogger,
+                                                bookieAddressResolver)
+                                        .withDefaultRack(NetworkTopology.DEFAULT_REGION_AND_RACK);
+                                perRegionPlacement.put(newRegion, newRegionPlacement);
+                            }
+                            newRegionPlacement.handleBookiesThatJoined(Collections.singleton(bookieAddress));
+                        }
+                    }
+                } catch (IllegalArgumentException | NetworkTopologyImpl.InvalidTopologyException e) {
+                    LOG.error("Failed to update bookie rack info: {} ", bookieAddress, e);
+                }
+            });
+        } finally {
+            rwLock.writeLock().unlock();
+        }
+    }
+
     @Override
     public RegionAwareEnsemblePlacementPolicy initialize(ClientConfiguration conf,
                                                          Optional<DNSToSwitchMapping> optionalDnsResolver,
