@@ -233,6 +233,7 @@ public class ZoneawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
         } else {
             String dnsResolverName = conf.getString(REPP_DNS_RESOLVER_CLASS, ScriptBasedMapping.class.getName());
             actualDNSResolver = ReflectionUtils.newInstance(dnsResolverName, DNSToSwitchMapping.class);
+            actualDNSResolver.setBookieAddressResolver(bookieAddressResolver);
             if (actualDNSResolver instanceof Configurable) {
                 ((Configurable) actualDNSResolver).setConf(conf);
             }
@@ -617,7 +618,7 @@ public class ZoneawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
         if (excludeZones.isEmpty()) {
             return "";
         }
-        StringBuilder excludedZonesString = new StringBuilder("~");
+        StringBuilder excludedZonesString = new StringBuilder(NetworkTopologyImpl.INVERSE);
         boolean firstZone = true;
         for (String excludeZone : excludeZones) {
             if (!firstZone) {
