@@ -54,7 +54,7 @@ import org.apache.bookkeeper.meta.LedgerManagerFactory;
 import org.apache.bookkeeper.meta.ZkLayoutManager;
 import org.apache.bookkeeper.meta.ZkLedgerUnderreplicationManager;
 import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
-import org.apache.bookkeeper.net.BookieSocketAddress;
+import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.proto.DataFormats.BookieServiceInfoFormat;
 import org.apache.bookkeeper.util.BookKeeperConstants;
 import org.apache.bookkeeper.util.ZkUtils;
@@ -158,7 +158,7 @@ public class ZKRegistrationManager implements RegistrationManager {
      * @param bookieId bookie id
      * @return
      */
-    public String getCookiePath(String bookieId) {
+    public String getCookiePath(BookieId bookieId) {
         return this.cookiePath + "/" + bookieId;
     }
 
@@ -217,7 +217,7 @@ public class ZKRegistrationManager implements RegistrationManager {
     }
 
     @Override
-    public void registerBookie(String bookieId, boolean readOnly,
+    public void registerBookie(BookieId bookieId, boolean readOnly,
                                BookieServiceInfo bookieServiceInfo) throws BookieException {
         if (!readOnly) {
             String regPath = bookieRegistrationPath + "/" + bookieId;
@@ -284,7 +284,8 @@ public class ZKRegistrationManager implements RegistrationManager {
         }
     }
 
-    private void doRegisterReadOnlyBookie(String bookieId, BookieServiceInfo bookieServiceInfo) throws BookieException {
+    private void doRegisterReadOnlyBookie(BookieId bookieId, BookieServiceInfo bookieServiceInfo)
+            throws BookieException {
         try {
             if (null == zk.exists(this.bookieReadonlyRegistrationPath, false)) {
                 try {
@@ -312,7 +313,7 @@ public class ZKRegistrationManager implements RegistrationManager {
     }
 
     @Override
-    public void unregisterBookie(String bookieId, boolean readOnly) throws BookieException {
+    public void unregisterBookie(BookieId bookieId, boolean readOnly) throws BookieException {
         String regPath;
         if (!readOnly) {
             regPath = bookieRegistrationPath + "/" + bookieId;
@@ -338,7 +339,7 @@ public class ZKRegistrationManager implements RegistrationManager {
     //
 
     @Override
-    public void writeCookie(String bookieId,
+    public void writeCookie(BookieId bookieId,
                             Versioned<byte[]> cookieData) throws BookieException {
         String zkPath = getCookiePath(bookieId);
         try {
@@ -365,16 +366,16 @@ public class ZKRegistrationManager implements RegistrationManager {
             Thread.currentThread().interrupt();
             throw new MetadataStoreException("Interrupted writing cookie for bookie " + bookieId, ie);
         } catch (NoNodeException nne) {
-            throw new CookieNotFoundException(bookieId);
+            throw new CookieNotFoundException(bookieId.toString());
         } catch (NodeExistsException nee) {
-            throw new CookieExistException(bookieId);
+            throw new CookieExistException(bookieId.toString());
         } catch (KeeperException e) {
             throw new MetadataStoreException("Failed to write cookie for bookie " + bookieId);
         }
     }
 
     @Override
-    public Versioned<byte[]> readCookie(String bookieId) throws BookieException {
+    public Versioned<byte[]> readCookie(BookieId bookieId) throws BookieException {
         String zkPath = getCookiePath(bookieId);
         try {
             Stat stat = zk.exists(zkPath, false);
@@ -383,19 +384,19 @@ public class ZKRegistrationManager implements RegistrationManager {
             LongVersion version = new LongVersion(stat.getVersion());
             return new Versioned<>(data, version);
         } catch (NoNodeException nne) {
-            throw new CookieNotFoundException(bookieId);
+            throw new CookieNotFoundException(bookieId.toString());
         } catch (KeeperException | InterruptedException e) {
             throw new MetadataStoreException("Failed to read cookie for bookie " + bookieId);
         }
     }
 
     @Override
-    public void removeCookie(String bookieId, Version version) throws BookieException {
+    public void removeCookie(BookieId bookieId, Version version) throws BookieException {
         String zkPath = getCookiePath(bookieId);
         try {
             zk.delete(zkPath, (int) ((LongVersion) version).getLongVersion());
         } catch (NoNodeException e) {
-            throw new CookieNotFoundException(bookieId);
+            throw new CookieNotFoundException(bookieId.toString());
         } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
             throw new MetadataStoreException("Interrupted deleting cookie for bookie " + bookieId, e);
@@ -514,10 +515,11 @@ public class ZKRegistrationManager implements RegistrationManager {
         try (RegistrationClient regClient = new ZKRegistrationClient(
             zk,
             ledgersRootPath,
-            null
+            null,
+            false
         )) {
             if (availableNodeExists) {
-                Collection<BookieSocketAddress> rwBookies = FutureUtils
+                Collection<BookieId> rwBookies = FutureUtils
                         .result(regClient.getWritableBookies(), EXCEPTION_FUNC).getValue();
                 if (rwBookies != null && !rwBookies.isEmpty()) {
                     log.error("Bookies are still up and connected to this cluster, "
@@ -527,7 +529,7 @@ public class ZKRegistrationManager implements RegistrationManager {
 
                 boolean readonlyNodeExists = null != zk.exists(bookieReadonlyRegistrationPath, false);
                 if (readonlyNodeExists) {
-                    Collection<BookieSocketAddress> roBookies = FutureUtils
+                    Collection<BookieId> roBookies = FutureUtils
                             .result(regClient.getReadOnlyBookies(), EXCEPTION_FUNC).getValue();
                     if (roBookies != null && !roBookies.isEmpty()) {
                         log.error("Readonly Bookies are still up and connected to this cluster, "
@@ -593,7 +595,7 @@ public class ZKRegistrationManager implements RegistrationManager {
     }
 
     @Override
-    public boolean isBookieRegistered(String bookieId) throws BookieException {
+    public boolean isBookieRegistered(BookieId bookieId) throws BookieException {
         String regPath = bookieRegistrationPath + "/" + bookieId;
         String readonlyRegPath = bookieReadonlyRegistrationPath + "/" + bookieId;
         try {
