@@ -33,6 +33,7 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Queue;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
@@ -94,9 +95,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
 
     private static class Lock {
         private final String lockZNode;
-        private final int ledgerZNodeVersion;
+        private final Optional<Integer> ledgerZNodeVersion;
 
-        Lock(String lockZNode, int ledgerZNodeVersion) {
+        Lock(String lockZNode, Optional<Integer> ledgerZNodeVersion) {
             this.lockZNode = lockZNode;
             this.ledgerZNodeVersion = ledgerZNodeVersion;
         }
@@ -105,13 +106,14 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             return lockZNode;
         }
 
-        int getLedgerZNodeVersion() {
+        Optional<Integer> getLedgerZNodeVersion() {
             return ledgerZNodeVersion;
         }
     }
     private final Map<Long, Lock> heldLocks = new ConcurrentHashMap<Long, Lock>();
     private final Pattern idExtractionPattern;
 
+    private final String rootPath;
     private final String basePath;
     private final String urLedgerPath;
     private final String urLockPath;
@@ -125,9 +127,10 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
     private final SubTreeCache subTreeCache;
 
     public ZkLedgerUnderreplicationManager(AbstractConfiguration conf, ZooKeeper zkc)
-            throws KeeperException, InterruptedException, ReplicationException.CompatibilityException {
+            throws UnavailableException, InterruptedException, ReplicationException.CompatibilityException {
         this.conf = conf;
-        basePath = getBasePath(ZKMetadataDriverBase.resolveZkLedgersRootPath(conf));
+        rootPath = ZKMetadataDriverBase.resolveZkLedgersRootPath(conf);
+        basePath = getBasePath(rootPath);
         layoutZNode = basePath + '/' + BookKeeperConstants.LAYOUT_ZNODE;
         urLedgerPath = basePath
                 + BookKeeperConstants.DEFAULT_ZK_LEDGERS_ROOT_PATH;
@@ -145,7 +148,11 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             }
         });
 
-        checkLayout();
+        try {
+            checkLayout();
+        } catch (KeeperException ke) {
+            throw ReplicationException.fromKeeperException("", ke);
+        }
     }
 
     public static String getBasePath(String rootPath) {
@@ -184,9 +191,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                 try {
                     zkc.create(layoutZNode, builder.build().toString().getBytes(UTF_8),
                                zkAcls, CreateMode.PERSISTENT);
+                    break;
                 } catch (KeeperException.NodeExistsException nne) {
                     // someone else managed to create it
-                    continue;
                 }
             } else {
                 byte[] layoutData = zkc.getData(layoutZNode, false, null);
@@ -280,7 +287,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             underreplicatedLedger.setReplicaList(replicaList);
             return underreplicatedLedger;
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while connecting zookeeper", ie);
@@ -389,24 +396,27 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
         try {
             Lock l = heldLocks.get(ledgerId);
             if (l != null) {
-                zkc.delete(getUrLedgerZnode(ledgerId), l.getLedgerZNodeVersion());
+                final Optional<Integer> ledgerZNodeVersion = l.getLedgerZNodeVersion();
+                if (ledgerZNodeVersion.isPresent()) {
+                    zkc.delete(getUrLedgerZnode(ledgerId), ledgerZNodeVersion.get());
 
-                try {
-                    // clean up the hierarchy
-                    String[] parts = getUrLedgerZnode(ledgerId).split("/");
-                    for (int i = 1; i <= 4; i++) {
-                        String[] p = Arrays.copyOf(parts, parts.length - i);
-                        String path = Joiner.on("/").join(p);
-                        Stat s = zkc.exists(path, null);
-                        if (s != null) {
-                            zkc.delete(path, s.getVersion());
+                    try {
+                        // clean up the hierarchy
+                        String[] parts = getUrLedgerZnode(ledgerId).split("/");
+                        for (int i = 1; i <= 4; i++) {
+                            String[] p = Arrays.copyOf(parts, parts.length - i);
+                            String path = Joiner.on("/").join(p);
+                            Stat s = zkc.exists(path, null);
+                            if (s != null) {
+                                zkc.delete(path, s.getVersion());
+                            }
                         }
+                    } catch (KeeperException.NotEmptyException nee) {
+                        // This can happen when cleaning up the hierarchy.
+                        // It's safe to ignore, it simply means another
+                        // ledger in the same hierarchy has been marked as
+                        // underreplicated.
                     }
-                } catch (KeeperException.NotEmptyException nee) {
-                    // This can happen when cleaning up the hierarchy.
-                    // It's safe to ignore, it simply means another
-                    // ledger in the same hierarchy has been marked as
-                    // underreplicated.
                 }
             }
         } catch (KeeperException.NoNodeException nne) {
@@ -417,7 +427,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             // znode in place, so the ledger is checked.
         } catch (KeeperException ke) {
             LOG.error("Error deleting underreplicated ledger znode", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -528,7 +538,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                     String lockPath = urLockPath + "/" + tryChild;
                     long ledgerId = getLedgerId(tryChild);
                     zkc.create(lockPath, LOCK_DATA, zkAcls, CreateMode.EPHEMERAL);
-                    heldLocks.put(ledgerId, new Lock(lockPath, stat.getVersion()));
+                    heldLocks.put(ledgerId, new Lock(lockPath, Optional.of(stat.getVersion())));
                     return ledgerId;
                 } catch (KeeperException.NodeExistsException nee) {
                     children.remove(tryChild);
@@ -571,7 +581,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
         try {
             return getLedgerToRereplicateFromHierarchy(urLedgerPath, 0);
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while connecting zookeeper", ie);
@@ -601,7 +611,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                 // nothing found, wait for a watcher to trigger
                 changedLatch.await();
             } catch (KeeperException ke) {
-                throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+                throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
             } catch (InterruptedException ie) {
                 Thread.currentThread().interrupt();
                 throw new ReplicationException.UnavailableException("Interrupted while connecting zookeeper", ie);
@@ -632,7 +642,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             // this is ok
         } catch (KeeperException ke) {
             LOG.error("Error deleting underreplicated ledger lock", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while connecting zookeeper", ie);
@@ -653,7 +663,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             // this is ok
         } catch (KeeperException ke) {
             LOG.error("Error deleting underreplicated ledger lock", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while connecting zookeeper", ie);
@@ -677,8 +687,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                     "AutoRecovery is already disabled!", ke);
         } catch (KeeperException ke) {
             LOG.error("Exception while stopping auto ledger re-replication", ke);
-            throw new ReplicationException.UnavailableException(
-                    "Exception while stopping auto ledger re-replication", ke);
+            throw ReplicationException.fromKeeperException("Exception while stopping auto ledger re-replication", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException(
@@ -701,8 +710,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                     "AutoRecovery is already enabled!", ke);
         } catch (KeeperException ke) {
             LOG.error("Exception while resuming ledger replication", ke);
-            throw new ReplicationException.UnavailableException(
-                    "Exception while resuming auto ledger re-replication", ke);
+            throw ReplicationException.fromKeeperException("Exception while resuming auto ledger re-replication", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException(
@@ -722,8 +730,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
         } catch (KeeperException ke) {
             LOG.error("Error while checking the state of "
                     + "ledger re-replication", ke);
-            throw new ReplicationException.UnavailableException(
-                    "Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException(
@@ -758,8 +765,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
         } catch (KeeperException ke) {
             LOG.error("Error while checking the state of "
                     + "ledger re-replication", ke);
-            throw new ReplicationException.UnavailableException(
-                    "Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException(
@@ -770,29 +776,39 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
     /**
      * Check whether the ledger is being replicated by any bookie.
      */
-    public static boolean isLedgerBeingReplicated(ZooKeeper zkc, String zkLedgersRootPath, long ledgerId)
-            throws KeeperException,
-            InterruptedException {
-        return zkc.exists(getUrLedgerLockZnode(getUrLockPath(zkLedgersRootPath), ledgerId), false) != null;
+    @Override
+    public boolean isLedgerBeingReplicated(long ledgerId) throws ReplicationException {
+        try {
+            return zkc.exists(getUrLedgerLockZnode(urLockPath, ledgerId), false) != null;
+        } catch (Exception e) {
+            throw new ReplicationException.UnavailableException("Failed to check ledger lock", e);
+        }
     }
 
     /**
      * Acquire the underreplicated ledger lock.
      */
-    public static void acquireUnderreplicatedLedgerLock(ZooKeeper zkc, String zkLedgersRootPath,
+    public static String acquireUnderreplicatedLedgerLock(ZooKeeper zkc, String zkLedgersRootPath,
         long ledgerId, List<ACL> zkAcls)
-            throws KeeperException, InterruptedException {
-        ZkUtils.createFullPathOptimistic(zkc, getUrLedgerLockZnode(getUrLockPath(zkLedgersRootPath), ledgerId),
-                LOCK_DATA, zkAcls, CreateMode.EPHEMERAL);
+            throws UnavailableException, InterruptedException {
+        try {
+            final String lockPath = getUrLedgerLockZnode(getUrLockPath(zkLedgersRootPath), ledgerId);
+            ZkUtils.createFullPathOptimistic(zkc, lockPath, LOCK_DATA, zkAcls, CreateMode.EPHEMERAL);
+            return lockPath;
+        } catch (KeeperException ke) {
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
+        }
     }
 
-    /**
-     * Release the underreplicated ledger lock if it exists.
-     */
-    public static void releaseUnderreplicatedLedgerLock(ZooKeeper zkc, String zkLedgersRootPath, long ledgerId)
-            throws InterruptedException, KeeperException {
-        if (isLedgerBeingReplicated(zkc, zkLedgersRootPath, ledgerId)) {
-            zkc.delete(getUrLedgerLockZnode(getUrLockPath(zkLedgersRootPath), ledgerId), -1);
+    @Override
+    public void acquireUnderreplicatedLedger(long ledgerId)
+            throws ReplicationException  {
+        try {
+            final String lockPath = acquireUnderreplicatedLedgerLock(zkc, rootPath, ledgerId, ZkUtils.getACLs(conf));
+            heldLocks.put(ledgerId, new Lock(lockPath, Optional.empty()));
+        } catch (Exception e) {
+            throw new ReplicationException.UnavailableException(
+                    "Failed to acquire underreplicated ledger lock for " + ledgerId, e);
         }
     }
 
@@ -811,7 +827,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             return false;
         } catch (KeeperException ke) {
             LOG.error("Error while initializing LostBookieRecoveryDelay", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -832,7 +848,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             }
         } catch (KeeperException ke) {
             LOG.error("Error while setting LostBookieRecoveryDelay ", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -847,7 +863,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             return Integer.parseInt(new String(data, UTF_8));
         } catch (KeeperException ke) {
             LOG.error("Error while getting LostBookieRecoveryDelay ", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -872,7 +888,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             }
         } catch (KeeperException ke) {
             LOG.error("Error while checking the state of lostBookieRecoveryDelay", ke);
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -893,7 +909,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             // this is ok.
         } catch (KeeperException e) {
             LOG.error("Error while getting ReplicationWorkerId rereplicating Ledger", e);
-            throw new ReplicationException.UnavailableException(
+            throw ReplicationException.fromKeeperException(
                     "Error while getting ReplicationWorkerId rereplicating Ledger", e);
         } catch (InterruptedException e) {
             LOG.error("Got interrupted while getting ReplicationWorkerId rereplicating Ledger", e);
@@ -922,7 +938,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                 zkc.create(checkAllLedgersCtimeZnode, checkAllLedgersFormatByteArray, zkAcls, CreateMode.PERSISTENT);
             }
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -943,7 +959,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             LOG.warn("checkAllLedgersCtimeZnode is not yet available");
             return -1;
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -969,7 +985,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                         CreateMode.PERSISTENT);
             }
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -990,7 +1006,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             LOG.warn("placementPolicyCheckCtimeZnode is not yet available");
             return -1;
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -1015,7 +1031,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
                 LOG.debug("setReplicasCheckCTime completed successfully");
             }
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
@@ -1035,7 +1051,7 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
             LOG.warn("replicasCheckCtimeZnode is not yet available");
             return -1;
         } catch (KeeperException ke) {
-            throw new ReplicationException.UnavailableException("Error contacting zookeeper", ke);
+            throw ReplicationException.fromKeeperException("Error contacting zookeeper", ke);
         } catch (InterruptedException ie) {
             Thread.currentThread().interrupt();
             throw new ReplicationException.UnavailableException("Interrupted while contacting zookeeper", ie);
