@@ -21,7 +21,6 @@ import com.google.common.annotations.VisibleForTesting;
 import io.netty.buffer.ByteBuf;
 import io.netty.channel.Channel;
 import io.netty.util.Recycler;
-
 import java.io.IOException;
 import java.util.concurrent.TimeUnit;
 
@@ -68,6 +67,7 @@ class WriteEntryProcessor extends PacketProcessorBase<ParsedAddRequest> implemen
                          requestProcessor.getRequestStats().getAddRequestStats());
             request.release();
             request.recycle();
+            recycle();
             return;
         }
 
@@ -101,8 +101,6 @@ class WriteEntryProcessor extends PacketProcessorBase<ParsedAddRequest> implemen
                       request.ledgerId, request.entryId, t.getMessage(), t);
             // some bad request which cause unexpected exception
             rc = BookieProtocol.EBADREQ;
-        } finally {
-            addData.release();
         }
 
         if (rc != BookieProtocol.EOK) {
@@ -112,6 +110,7 @@ class WriteEntryProcessor extends PacketProcessorBase<ParsedAddRequest> implemen
                          ResponseBuilder.buildErrorResponse(rc, request),
                          requestProcessor.getRequestStats().getAddRequestStats());
             request.recycle();
+            recycle();
         }
     }
 
