@@ -131,7 +131,7 @@ class WriteEntryProcessorV3 extends PacketProcessorBaseV3 {
             if (logger.isDebugEnabled()) {
                 logger.debug("Operation rejected while writing {}", request, e);
             }
-            status = StatusCode.EIO;
+            status = StatusCode.ETOOMANYREQUESTS;
         } catch (IOException e) {
             logger.error("Error writing entry:{} to ledger:{}",
                     entryId, ledgerId, e);
@@ -162,6 +162,8 @@ class WriteEntryProcessorV3 extends PacketProcessorBaseV3 {
 
     @Override
     public void safeRun() {
+        requestProcessor.getRequestStats().getWriteThreadQueuedLatency()
+                .registerSuccessfulEvent(MathUtils.elapsedNanos(enqueueNanos), TimeUnit.NANOSECONDS);
         AddResponse addResponse = getAddResponse();
         if (null != addResponse) {
             // This means there was an error and we should send this back.
