@@ -30,6 +30,7 @@ public interface ReplicationStats {
     String AUDITOR_SCOPE = "auditor";
     String ELECTION_ATTEMPTS = "election_attempts";
     String NUM_UNDER_REPLICATED_LEDGERS = "NUM_UNDER_REPLICATED_LEDGERS";
+    String UNDER_REPLICATED_LEDGERS_TOTAL_SIZE = "UNDER_REPLICATED_LEDGERS_TOTAL_SIZE";
     String URL_PUBLISH_TIME_FOR_LOST_BOOKIE = "URL_PUBLISH_TIME_FOR_LOST_BOOKIE";
     String BOOKIE_TO_LEDGERS_MAP_CREATION_TIME = "BOOKIE_TO_LEDGERS_MAP_CREATION_TIME";
     String CHECK_ALL_LEDGERS_TIME = "CHECK_ALL_LEDGERS_TIME";
@@ -58,6 +59,8 @@ public interface ReplicationStats {
     String NUM_BYTES_READ = "NUM_BYTES_READ";
     String NUM_ENTRIES_WRITTEN = "NUM_ENTRIES_WRITTEN";
     String NUM_BYTES_WRITTEN = "NUM_BYTES_WRITTEN";
+    String READ_DATA_LATENCY = "READ_DATA_LATENCY";
+    String WRITE_DATA_LATENCY = "WRITE_DATA_LATENCY";
     String REPLICATE_EXCEPTION = "exceptions";
     String NUM_DEFER_LEDGER_LOCK_RELEASE_OF_FAILED_LEDGER = "NUM_DEFER_LEDGER_LOCK_RELEASE_OF_FAILED_LEDGER";
     String NUM_ENTRIES_UNABLE_TO_READ_FOR_REPLICATION = "NUM_ENTRIES_UNABLE_TO_READ_FOR_REPLICATION";
