@@ -557,7 +557,7 @@ public class ConcurrentLongLongHashMap {
             int removedCount = 0;
             try {
                 // Go through all the buckets for this section
-                for (int bucket = 0; bucket < table.length; bucket += 2) {
+                for (int bucket = 0; size > 0 && bucket < table.length; bucket += 2) {
                     long storedKey = table[bucket];
 
                     if (storedKey != DeletedKey && storedKey != EmptyKey) {
@@ -582,7 +582,7 @@ public class ConcurrentLongLongHashMap {
             int removedCount = 0;
             try {
                 // Go through all the buckets for this section
-                for (int bucket = 0; bucket < table.length; bucket += 2) {
+                for (int bucket = 0; size > 0 && bucket < table.length; bucket += 2) {
                     long storedKey = table[bucket];
                     long storedValue = table[bucket + 1];
 
@@ -731,4 +731,20 @@ public class ConcurrentLongLongHashMap {
             throw new IllegalArgumentException("Keys and values must be >= 0");
         }
     }
+
+    @Override
+    public String toString() {
+        StringBuilder sb = new StringBuilder();
+        sb.append("ConcurrentLongLongHashMap{");
+
+        int headerLen = sb.length();
+        forEach((k, v) -> {
+                sb.append(k).append(" => ").append(v).append(", ");
+            });
+        if (sb.length() > headerLen) {
+            sb.setLength(sb.length() - 2);
+        }
+        sb.append("}");
+        return sb.toString();
+    }
 }
