@@ -35,7 +35,7 @@ import org.apache.bookkeeper.client.BookKeeper.DigestType;
 import org.apache.bookkeeper.common.testing.annotations.FlakyTest;
 import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.conf.ServerConfiguration;
-import org.apache.bookkeeper.net.BookieSocketAddress;
+import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.slf4j.Logger;
@@ -57,7 +57,7 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
             super(conf);
         }
 
-        void blockUntilBookieWeightIs(BookieSocketAddress bookie, Optional<Long> target) throws InterruptedException {
+        void blockUntilBookieWeightIs(BookieId bookie, Optional<Long> target) throws InterruptedException {
             long startMsecs = System.currentTimeMillis();
             Optional<Long> freeDiskSpace = Optional.empty();
             while (System.currentTimeMillis() < (startMsecs + MS_WEIGHT_UPDATE_TIMEOUT)) {
@@ -95,7 +95,7 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         bsConfs.add(conf);
         BookieServer server = startBookie(conf, bookieWithCustomFreeDiskSpace);
         bs.add(server);
-        client.blockUntilBookieWeightIs(server.getLocalAddress(), Optional.of(initialFreeDiskSpace));
+        client.blockUntilBookieWeightIs(server.getBookieId(), Optional.of(initialFreeDiskSpace));
         if (useFinal == null) {
             ready.set(true);
         }
@@ -114,7 +114,7 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
             BookieServer bookie, final long freeDiskSpace)
             throws Exception {
         for (int i = 0; i < bs.size(); i++) {
-            if (bs.get(i).getLocalAddress().equals(bookie.getLocalAddress())) {
+            if (bs.get(i).getBookieId().equals(bookie.getBookieId())) {
                 return replaceBookieWithCustomFreeDiskSpaceBookie(client, i, freeDiskSpace);
             }
         }
@@ -125,7 +125,7 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
             BookKeeperCheckInfoReader client,
             int bookieIdx, long initialFreeDiskSpace,
              long finalFreeDiskSpace, AtomicBoolean useFinal) throws Exception {
-        BookieSocketAddress addr = bs.get(bookieIdx).getLocalAddress();
+        BookieId addr = bs.get(bookieIdx).getBookieId();
         LOG.info("Killing bookie {}", addr);
         ServerConfiguration conf = killBookieAndWaitForZK(bookieIdx);
         client.blockUntilBookieWeightIs(addr, Optional.empty());
@@ -155,14 +155,14 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
                 replaceBookieWithCustomFreeDiskSpaceBookie(client, 0, multiple * freeDiskSpace);
             }
         }
-        Map<BookieSocketAddress, Integer> m = new HashMap<BookieSocketAddress, Integer>();
+        Map<BookieId, Integer> m = new HashMap<BookieId, Integer>();
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -170,12 +170,12 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight(the last 2 bookies) are chosen 3X as often as the median;
         // since the number of ledgers created is small (2000), we allow a range of 2X to 4X instead of the exact 3X
         for (int i = 0; i < numBookies - 2; i++) {
-            double ratio1 = (double) m.get(bs.get(numBookies - 2).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio1 = (double) m.get(bs.get(numBookies - 2).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                     Math.abs(ratio1 - multiple) < 1);
-            double ratio2 = (double) m.get(bs.get(numBookies - 1).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio2 = (double) m.get(bs.get(numBookies - 1).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                     Math.abs(ratio2 - multiple) < 1);
         }
@@ -205,14 +205,14 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
                 replaceBookieWithCustomFreeDiskSpaceBookie(client, 0, multiple * freeDiskSpace);
             }
         }
-        Map<BookieSocketAddress, Integer> m = new HashMap<BookieSocketAddress, Integer>();
+        Map<BookieId, Integer> m = new HashMap<BookieId, Integer>();
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -220,12 +220,12 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight(the last 2 bookies) are chosen 3X as often as the median;
         // since the number of ledgers created is small (2000), we allow a range of 2X to 4X instead of the exact 3X
         for (int i = 0; i < numBookies - 2; i++) {
-            double ratio1 = (double) m.get(bs.get(numBookies - 2).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio1 = (double) m.get(bs.get(numBookies - 2).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                     Math.abs(ratio1 - multiple) < 1);
-            double ratio2 = (double) m.get(bs.get(numBookies - 1).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio2 = (double) m.get(bs.get(numBookies - 1).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                     Math.abs(ratio2 - multiple) < 1);
         }
@@ -243,11 +243,11 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         server4 = replaceBookieWithCustomFreeDiskSpaceBookie(client, server4, freeDiskSpace);
 
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -255,16 +255,16 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight(the last 2 bookies) are chosen 3X as often as the median;
         // since the number of ledgers created is small (2000), we allow a range of 2X to 4X instead of the exact 3X
         for (int i = 0; i < numBookies; i++) {
-            if (server1.getLocalAddress().equals(bs.get(i).getLocalAddress())
-                    || server2.getLocalAddress().equals(bs.get(i).getLocalAddress())) {
+            if (server1.getBookieId().equals(bs.get(i).getBookieId())
+                    || server2.getBookieId().equals(bs.get(i).getBookieId())) {
                 continue;
             }
-            double ratio1 = (double) m.get(server1.getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio1 = (double) m.get(server1.getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                     Math.abs(ratio1 - multiple) < 1);
-            double ratio2 = (double) m.get(server2.getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio2 = (double) m.get(server2.getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                     Math.abs(ratio2 - multiple) < 1);
         }
@@ -295,33 +295,33 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
                 replaceBookieWithCustomFreeDiskSpaceBookie(client, 0, multiple * freeDiskSpace);
             }
         }
-        Map<BookieSocketAddress, Integer> m = new HashMap<BookieSocketAddress, Integer>();
+        Map<BookieId, Integer> m = new HashMap<BookieId, Integer>();
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
 
         // make sure that bookies with higher weight are chosen 3X as often as the median;
         // since the number of ledgers is small (2000), there may be variation
-        double ratio1 = (double) m.get(bs.get(numBookies - 2).getLocalAddress())
-            / (double) m.get(bs.get(0).getLocalAddress());
+        double ratio1 = (double) m.get(bs.get(numBookies - 2).getBookieId())
+            / (double) m.get(bs.get(0).getBookieId());
         assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                 Math.abs(ratio1 - multiple) < 1);
-        double ratio2 = (double) m.get(bs.get(numBookies - 1).getLocalAddress())
-            / (double) m.get(bs.get(1).getLocalAddress());
+        double ratio2 = (double) m.get(bs.get(numBookies - 1).getBookieId())
+            / (double) m.get(bs.get(1).getBookieId());
         assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                 Math.abs(ratio2 - multiple) < 1);
 
         // Bring down the 2 bookies that had higher weight; after this the allocation to all
         // the remaining bookies should be uniform
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
         BookieServer server1 = bs.get(numBookies - 2);
         BookieServer server2 = bs.get(numBookies - 1);
@@ -330,24 +330,24 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
 
         // make sure that bookies with higher weight are chosen 3X as often as the median;
         for (int i = 0; i < numBookies - 3; i++) {
-            double delta = Math.abs((double) m.get(bs.get(i).getLocalAddress())
-                    - (double) m.get(bs.get(i + 1).getLocalAddress()));
-            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getLocalAddress());
+            double delta = Math.abs((double) m.get(bs.get(i).getBookieId())
+                    - (double) m.get(bs.get(i + 1).getBookieId()));
+            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getBookieId());
             // the deviation should be less than 30%
             assertTrue("Weigheted placement is not honored: " + delta, delta <= 30);
         }
         // since the following 2 bookies were down, they shouldn't ever be selected
-        assertTrue("Weigheted placement is not honored" + m.get(server1.getLocalAddress()),
-                m.get(server1.getLocalAddress()) == 0);
-        assertTrue("Weigheted placement is not honored" + m.get(server2.getLocalAddress()),
-                m.get(server2.getLocalAddress()) == 0);
+        assertTrue("Weigheted placement is not honored" + m.get(server1.getBookieId()),
+                m.get(server1.getBookieId()) == 0);
+        assertTrue("Weigheted placement is not honored" + m.get(server2.getBookieId()),
+                m.get(server2.getBookieId()) == 0);
 
         client.close();
     }
@@ -375,14 +375,14 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // let the last two bookies be down initially
         ServerConfiguration conf1 = killBookieAndWaitForZK(numBookies - 1);
         ServerConfiguration conf2 = killBookieAndWaitForZK(numBookies - 2);
-        Map<BookieSocketAddress, Integer> m = new HashMap<BookieSocketAddress, Integer>();
+        Map<BookieId, Integer> m = new HashMap<BookieId, Integer>();
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -390,9 +390,9 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight are chosen 3X as often as the median;
         // since the number of ledgers is small (2000), there may be variation
         for (int i = 0; i < numBookies - 3; i++) {
-            double delta = Math.abs((double) m.get(bs.get(i).getLocalAddress())
-                    - (double) m.get(bs.get(i + 1).getLocalAddress()));
-            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getLocalAddress());
+            double delta = Math.abs((double) m.get(bs.get(i).getBookieId())
+                    - (double) m.get(bs.get(i + 1).getBookieId()));
+            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getBookieId());
             // the deviation should be less than 30%
             assertTrue("Weigheted placement is not honored: " + delta, delta <= 30);
         }
@@ -402,11 +402,11 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         restartBookie(client, conf2, multiple * freeDiskSpace, multiple * freeDiskSpace, null);
 
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -414,12 +414,12 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight(the last 2 bookies) are chosen 3X as often as the median;
         // since the number of ledgers created is small (2000), we allow a range of 2X to 4X instead of the exact 3X
         for (int i = 0; i < numBookies - 2; i++) {
-            double ratio1 = (double) m.get(bs.get(numBookies - 2).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio1 = (double) m.get(bs.get(numBookies - 2).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                     Math.abs(ratio1 - multiple) < 1);
-            double ratio2 = (double) m.get(bs.get(numBookies - 1).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio2 = (double) m.get(bs.get(numBookies - 1).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                     Math.abs(ratio2 - multiple) < 1);
         }
@@ -455,22 +455,22 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
                         client, 0, freeDiskSpace, multiple * freeDiskSpace, useHigherValue);
             }
         }
-        Map<BookieSocketAddress, Integer> m = new HashMap<BookieSocketAddress, Integer>();
+        Map<BookieId, Integer> m = new HashMap<BookieId, Integer>();
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
 
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
 
         for (int i = 0; i < numBookies - 1; i++) {
-            double delta = Math.abs((double) m.get(bs.get(i).getLocalAddress())
-                    - (double) m.get(bs.get(i + 1).getLocalAddress()));
-            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getLocalAddress());
+            double delta = Math.abs((double) m.get(bs.get(i).getBookieId())
+                    - (double) m.get(bs.get(i + 1).getBookieId()));
+            delta = (delta * 100) / (double) m.get(bs.get(i + 1).getBookieId());
             assertTrue("Weigheted placement is not honored: " + delta, delta <= 30); // the deviation should be <30%
         }
 
@@ -480,18 +480,18 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         Thread.sleep(updateIntervalSecs * 1000);
         for (int i = 0; i < numBookies; i++) {
             if (i < numBookies - 2) {
-                client.blockUntilBookieWeightIs(bs.get(i).getLocalAddress(), Optional.of(freeDiskSpace));
+                client.blockUntilBookieWeightIs(bs.get(i).getBookieId(), Optional.of(freeDiskSpace));
             } else {
-                client.blockUntilBookieWeightIs(bs.get(i).getLocalAddress(), Optional.of(freeDiskSpace * multiple));
+                client.blockUntilBookieWeightIs(bs.get(i).getBookieId(), Optional.of(freeDiskSpace * multiple));
             }
         }
 
         for (BookieServer b : bs) {
-            m.put(b.getLocalAddress(), 0);
+            m.put(b.getBookieId(), 0);
         }
         for (int i = 0; i < 2000; i++) {
             LedgerHandle lh = client.createLedger(3, 3, DigestType.CRC32, "testPasswd".getBytes());
-            for (BookieSocketAddress b : lh.getLedgerMetadata().getEnsembleAt(0)) {
+            for (BookieId b : lh.getLedgerMetadata().getEnsembleAt(0)) {
                 m.put(b, m.get(b) + 1);
             }
         }
@@ -499,12 +499,12 @@ public class BookKeeperDiskSpaceWeightedLedgerPlacementTest extends BookKeeperCl
         // make sure that bookies with higher weight(the last 2 bookies) are chosen 3X as often as the median;
         // since the number of ledgers created is small (2000), we allow a range of 2X to 4X instead of the exact 3X
         for (int i = 0; i < numBookies - 2; i++) {
-            double ratio1 = (double) m.get(bs.get(numBookies - 2).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio1 = (double) m.get(bs.get(numBookies - 2).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio1 - multiple),
                     Math.abs(ratio1 - multiple) < 1);
-            double ratio2 = (double) m.get(bs.get(numBookies - 1).getLocalAddress())
-                / (double) m.get(bs.get(i).getLocalAddress());
+            double ratio2 = (double) m.get(bs.get(numBookies - 1).getBookieId())
+                / (double) m.get(bs.get(i).getBookieId());
             assertTrue("Weigheted placement is not honored: " + Math.abs(ratio2 - multiple),
                     Math.abs(ratio2 - multiple) < 1);
         }
