@@ -37,7 +37,6 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
-import java.util.UUID;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Future;
 import java.util.concurrent.SynchronousQueue;
@@ -62,6 +61,7 @@ import org.apache.bookkeeper.replication.Auditor;
 import org.apache.bookkeeper.replication.AutoRecoveryMain;
 import org.apache.bookkeeper.replication.ReplicationException.CompatibilityException;
 import org.apache.bookkeeper.replication.ReplicationException.UnavailableException;
+import org.apache.bookkeeper.replication.ReplicationWorker;
 import org.apache.bookkeeper.util.IOUtils;
 import org.apache.bookkeeper.util.PortManager;
 import org.apache.commons.io.FileUtils;
@@ -98,6 +98,7 @@ public abstract class BookKeeperClusterTestCase {
     protected final List<BookieServer> bs = new LinkedList<BookieServer>();
     protected final List<ServerConfiguration> bsConfs = new LinkedList<ServerConfiguration>();
     private final Map<BookieId, TestStatsProvider> bsLoggers = new HashMap<>();
+
     protected int numBookies;
     protected BookKeeperTestClient bkc;
     protected boolean useUUIDasBookieId = true;
@@ -643,12 +644,6 @@ public abstract class BookKeeperClusterTestCase {
     public int startNewBookie()
             throws Exception {
         ServerConfiguration conf = newServerConfiguration();
-
-        // use a random BookieId
-        if (useUUIDasBookieId) {
-            conf.setBookieId(UUID.randomUUID().toString());
-        }
-
         bsConfs.add(conf);
         LOG.info("Starting new bookie on port: {}", conf.getBookiePort());
         BookieServer server = startBookie(conf);
@@ -826,7 +821,13 @@ public abstract class BookKeeperClusterTestCase {
         while (System.nanoTime() < timeoutAt) {
             for (AutoRecoveryMain p : autoRecoveryProcesses.values()) {
                 Auditor a = p.getAuditor();
-                if (a != null) {
+                ReplicationWorker replicationWorker = p.getReplicationWorker();
+                if (a != null && a.isRunning() && replicationWorker != null && replicationWorker.isRunning()) {
+                    int deathWatchInterval = baseConf.getDeathWatchInterval();
+                    Thread.sleep(deathWatchInterval + 1000);
+                }
+
+                if (a != null && a.isRunning() && replicationWorker != null && replicationWorker.isRunning()) {
                     return a;
                 }
             }
@@ -835,6 +836,10 @@ public abstract class BookKeeperClusterTestCase {
         throw new Exception("No auditor found");
     }
 
+    public AutoRecoveryMain getAutoRecovery(BookieServer bServer) throws Exception {
+        return autoRecoveryProcesses.get(bServer);
+    }
+
     /**
      * Check whether the InetSocketAddress was created using a hostname or an IP
      * address. Represent as 'hostname/IPaddress' if the InetSocketAddress was
