@@ -44,7 +44,6 @@ import org.apache.bookkeeper.auth.AuthToken;
 import org.apache.bookkeeper.auth.BookieAuthProvider;
 import org.apache.bookkeeper.auth.ClientAuthProvider;
 import org.apache.bookkeeper.client.BKException;
-import org.apache.bookkeeper.client.BKException.BKUnauthorizedAccessException;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.client.BookKeeper.DigestType;
 import org.apache.bookkeeper.client.BookKeeperAdmin;
@@ -325,7 +324,7 @@ public class TestTLS extends BookKeeperClusterTestCase {
                 LedgerEntry e = entries.nextElement();
                 assertTrue("Entry contents incorrect", Arrays.equals(e.getEntry(), testEntry));
             }
-            BookKeeperAdmin admin = new BookKeeperAdmin(client);
+            BookKeeperAdmin admin = new BookKeeperAdmin(client, baseClientConf);
             return admin.getLedgerMetadata(lh);
         }
     }
@@ -896,7 +895,8 @@ public class TestTLS extends BookKeeperClusterTestCase {
                     .append("bookie_")
                     .append(bookie.getBookieId().toString()
                     .replace('.', '_')
-                    .replace('-', '_'))
+                    .replace('-', '_')
+                    .replace(":", "_"))
                     .append(".");
 
             // check stats on TLS enabled client
@@ -997,7 +997,8 @@ public class TestTLS extends BookKeeperClusterTestCase {
                 .append("bookie_")
                 .append(bookie.getBookieId().toString()
                         .replace('.', '_')
-                        .replace('-', '_'))
+                        .replace('-', '_')
+                        .replace(":", "_"))
                 .append(".");
 
         assertEquals("TLS handshake failure expected", 1,
@@ -1020,7 +1021,7 @@ public class TestTLS extends BookKeeperClusterTestCase {
         try {
             testClient(clientConf, numBookies);
             fail("should have failed with unauthorized exception");
-        } catch (BKUnauthorizedAccessException e) {
+        } catch (BKException.BKNotEnoughBookiesException e) {
             // Ok.
         }
     }
