@@ -21,11 +21,17 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertSame;
 
+import io.netty.buffer.ByteBuf;
+import io.netty.buffer.ByteBufAllocator;
+import java.io.StringWriter;
+import java.util.Collections;
+import java.util.HashMap;
+
 import lombok.Cleanup;
 import org.apache.bookkeeper.stats.Counter;
-import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.stats.StatsLogger;
 import org.apache.commons.configuration.PropertiesConfiguration;
+import org.junit.Assert;
 import org.junit.Test;
 
 /**
@@ -33,25 +39,6 @@ import org.junit.Test;
  */
 public class TestPrometheusMetricsProvider {
 
-    @Test
-    public void testCache() {
-        PrometheusMetricsProvider provider = new PrometheusMetricsProvider();
-
-        StatsLogger statsLogger =  provider.getStatsLogger("test");
-
-        OpStatsLogger opStatsLogger1 = statsLogger.getOpStatsLogger("optest");
-        OpStatsLogger opStatsLogger2 = statsLogger.getOpStatsLogger("optest");
-        assertSame(opStatsLogger1, opStatsLogger2);
-
-        Counter counter1 = statsLogger.getCounter("countertest");
-        Counter counter2 = statsLogger.getCounter("countertest");
-        assertSame(counter1, counter2);
-
-        StatsLogger scope1 = statsLogger.scope("scopetest");
-        StatsLogger scope2 = statsLogger.scope("scopetest");
-        assertSame(scope1, scope2);
-    }
-
     @Test
     public void testStartNoHttp() {
         PropertiesConfiguration config = new PropertiesConfiguration();
@@ -106,7 +93,7 @@ public class TestPrometheusMetricsProvider {
 
     @Test
     public void testCounter() {
-        LongAdderCounter counter = new LongAdderCounter();
+        LongAdderCounter counter = new LongAdderCounter(Collections.emptyMap());
         long value = counter.get();
         assertEquals(0L, value);
         counter.inc();
@@ -130,4 +117,39 @@ public class TestPrometheusMetricsProvider {
         assertEquals(1, provider.counters.size());
     }
 
+    @Test
+    public void testJvmDirectMemoryMetrics() throws Exception {
+        PropertiesConfiguration config = new PropertiesConfiguration();
+        config.setProperty(PrometheusMetricsProvider.PROMETHEUS_STATS_HTTP_ENABLE, true);
+        config.setProperty(PrometheusMetricsProvider.PROMETHEUS_STATS_HTTP_PORT, 0);
+        config.setProperty(PrometheusMetricsProvider.PROMETHEUS_STATS_HTTP_ADDRESS, "127.0.0.1");
+        ByteBuf byteBuf = ByteBufAllocator.DEFAULT.directBuffer(25);
+        PrometheusMetricsProvider provider = new PrometheusMetricsProvider();
+        try {
+            provider.start(config);
+            assertNotNull(provider.server);
+            StringWriter writer = new StringWriter();
+            provider.writeAllMetrics(writer);
+            String s = writer.toString();
+            String[] split = s.split(System.lineSeparator());
+            HashMap<String, String> map = new HashMap<>();
+            for (String str : split) {
+                String[] aux = str.split(" ");
+                map.put(aux[0], aux[1]);
+            }
+            String directBytesMax = map.get("jvm_memory_direct_bytes_max{}");
+            Assert.assertNotNull(directBytesMax);
+            Assert.assertNotEquals("Nan", directBytesMax);
+            Assert.assertNotEquals("-1", directBytesMax);
+            String directBytesUsed = map.get("jvm_memory_direct_bytes_used{}");
+            Assert.assertNotNull(directBytesUsed);
+            Assert.assertNotEquals("Nan", directBytesUsed);
+            Assert.assertTrue(Double.parseDouble(directBytesUsed) > 25);
+            // ensure byteBuffer doesn't gc
+            byteBuf.release();
+        } finally {
+            provider.stop();
+        }
+    }
+
 }
