@@ -30,6 +30,7 @@ import com.google.protobuf.TextFormat;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.PooledByteBufAllocator;
 import io.netty.buffer.Unpooled;
+import io.netty.util.ReferenceCountUtil;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
@@ -490,7 +491,7 @@ class MVCCStoreImpl<K, V> extends RocksdbKVStore<K, V> implements MVCCStore<K, V
         try {
             record = getKeyRecord(key, rawKey);
         } catch (StateStoreRuntimeException e) {
-            rawValBuf.release();
+            ReferenceCountUtil.release(rawValBuf);
             throw e;
         }
 
@@ -965,7 +966,7 @@ class MVCCStoreImpl<K, V> extends RocksdbKVStore<K, V> implements MVCCStore<K, V
 
         // raw key
         byte[] rawKey = (null != key) ? keyCoder.encode(key) : NULL_START_KEY;
-
+        byte[] rawEndKey = NULL_END_KEY;
         if (null == endKey) {
             // point lookup
             MVCCRecord record = getKeyRecord(key, rawKey);
@@ -988,8 +989,9 @@ class MVCCStoreImpl<K, V> extends RocksdbKVStore<K, V> implements MVCCStore<K, V
                     record.recycle();
                 }
             }
+        } else {
+            rawEndKey = keyCoder.encode(endKey);
         }
-        byte[] rawEndKey = (null != endKey) ? keyCoder.encode(endKey) : NULL_END_KEY;
         Pair<byte[], byte[]> realRange = getRealRange(rawKey, rawEndKey);
         rawKey = realRange.getLeft();
         rawEndKey = realRange.getRight();
